from typing import Optional, List
from pydantic import BaseModel, Field, ValidationError

# define fields of resource object
field_defs = {
    "id": Field(
        None,
        title='id',
        description='string of N digits.'
                    'unique per OpsRamp instance (a.k.a OpsRamp POD).'
                    'default field in any OpsQL queries: yes.'
    ),
    "subject": Field (
        None,
        title = 'subject',
        description='Summary of what the alert is about.'
                    'E.g. SNMP request timeout on server-abc.'
                    'default field in any OpsQL queries: yes.'
    ),
    "description": Field (
        None,
        title = 'description',
        description='Details about the alert  '
                    'E.g. SNMP requests are timed out for the following reasons...'
    ),
    "metric":  Field(
        None,
        title='metric',
        description='metric or event that triggered the alert.'
                    'E.g. snmp.response.timeout.'
                    'default field in any OpsQL queries: yes.'
    ),
    "component": Field(
        None,
        title='metric components',
        description='TBD'
                    'E.g. TBD'
    ),
    "repeatCount": Field(
        None,
        title='repeatCount',
        description='Number of times the same alert has re-triggered since it first went to open state'
                    'E.g. repeat count = 10'
    ),
    "isAvailabilityAlert" : Field(
        None,
        title='isAvailabilityAlert',
        description='Does this alert denote availability of the monitored resource'
                    'E.g. isAvailabilityAlert: true'
    ),
    "alertType": Field(
        None,
        title='alertType',
        description='type of alert possible values:'
                    'agent, gateway, change detection, forecast'
                    'maintenance, scheduled maintenance'
    ),
    "objectId": Field(
        None,
        title='entityId',
        description='Id of the object associated with the alert.'
                    'this can be: a client id or resource id.'
                    'default field in any OpsQL queries: yes.'
    ),
    "objectName": Field(
        None,
        title='objectName',
        description='Name of the object associated with the alert.'
                    'this can be: a client name or resource name.'
                    'default field in any OpsQL queries: yes.'
    ),
    "objectType": Field(
        None,
        title='objectType',
        description='Type of the object associated with the alert.'
                    'e.g.: client-abc or Server.'
                    'default field in any OpsQL queries: yes.'
    ),
    "ipAddress": Field(
        None,
        title='ipAddress',
        description='ipAddress of the resource associated with the alert'
                    'e.g.: 10.10.10.1'
    ),
    "triggeredTime": Field(
        None,
        title='triggeredTime',
        description='timestamp of when the alert was triggered at the source - agent, gateway, ...'
                    'Format: In ISO_8601 format, always in UTC: 2020-12-08T10:30:15+0000'
    ),
    "createdTime": Field(
        None,
        title='createdTime',
        description='timestamp of when the alert was received by the OpsRamp cloud.'
                    'Format: In ISO_8601 format, always in UTC: 2020-12-08T10:30:15+0000.'
                    'default field in any OpsQL queries: yes.'
    ),
    "alertCriticalWarningDuration": Field(
        None,
        title='alertCriticalWarningDuration',
        description='how long the alert was in a crtical or warning state in minutes'
                    'e.g. 12 minutes'
    ),
    "updatedTime": Field(
        None,
        title='updatedTime',
        description='timestamp of when the alert was last updated'
                    'Format: In ISO_8601 format, always in UTC: 2020-12-08T10:30:15+0000'
    ),
    "currentState": Field(
        None,
        title='currentState',
        description='current severity of the condition behind the alert.'
                    'this can be OK, warning, or critical.'
                    'default field in any OpsQL queries: yes.'
    ),
    "previousState": Field(
        None,
        title='previousState',
        description='previous severity of the condition behind the alert'
                    'this can be OK, warning, or critical'
    ),
    "status": Field(
        None,
        title='status',
        description='action taken on the alert'
                    'this can be open, acknowleged, suppressed, ticketed, closed'
    ),
    "inferenceId": Field(
        None,
        title='inferenceId',
        description='id of the inference to which this alert has been correlated'
                    'e.g. 2531518'
    ),
    "incidentId": Field(
        None,
        title='incidentId',
        description='id the incident that was created from this alert'
                    'e.g. INC0006765593'
    ),
    "resourceMaintenanceWindowId": Field(
        None,
        title='resourceMaintenanceWindowId',
        description='scheduled maintenance Window Id'
    ),
    "clientId": Field(
        None,
        title='clientId',
        description='id of client corresponding to the resource associated with the alert'
    ),
    "clientName": Field(
        None,
        title='clientName',
        description='Name of client corresponding to the resource associated with the alert'
    ),
    "source": Field(
        None,
        title='source',
        description='Denotes what type of system generated the alert - e.g. OpsRamp, Splunk, etc.'
    ),
    "externalAlertId": Field(
        None,
        title='externalAlertId',
        description='For alerts that come from third-party source,' 
        'this denotes the id of the alert in that third-party tool'
    ),
    "comments": Field(
        None,
        title='comments',
        description='comments added by a user to the alert'
                    'e.g. create an incident if this alert remains open for longer than 20 minutes'
    )
}



class PostedAlert(BaseModel):
    subject : str = field_defs["subject"]
    description : str = field_defs["description"]
    metric : Optional[str] = field_defs["metric"]
    component: Optional[str] = field_defs["component"]
    isAvailabilityAlert: Optional[str] = field_defs["isAvailabilityAlert"]
    alertType: Optional[str] = field_defs["alertType"]
    objectId: str = field_defs["objectId"]
    triggeredTime: Optional[str] = field_defs["triggeredTime"]
    currentState: str = field_defs["currentState"]
    source: str = field_defs["source"]
    externalAlertId: str = field_defs["externalAlertId"]
    comments: str = field_defs["comments"]



# The class is the domain object that represents the alert ingested or generated by OpsRamp.
# This is what you get when you do GET /alerts/{id}
class Alert(BaseModel):
    id : str = field_defs["id"]
    subject : str = field_defs["subject"]
    description : str = field_defs["description"]
    metric : Optional[str] = field_defs["metric"]
    component: Optional[str] = field_defs["component"]
    repeatCount: Optional[int] = field_defs["repeatCount"]
    alertType: Optional[str] = field_defs["alertType"]
    objectId: str = field_defs["objectId"]
    objectName: str = field_defs["objectName"]
    objectType: str = field_defs["objectType"]
    ipAddress: Optional[str] = field_defs["ipAddress"]
    triggeredTime: Optional[str] = field_defs["triggeredTime"]
    createdTime: str = field_defs["createdTime"]
    alertCriticalWarningDuration: str = field_defs["alertCriticalWarningDuration"]
    updatedTime: str = field_defs["updatedTime"]
    currentState: str = field_defs["currentState"]
    previousState: str = field_defs["previousState"]
    status: str = field_defs["status"]
    inferenceId: str = field_defs["inferenceId"]
    incidentId: str = field_defs["incidentId"]
    resourceMaintenanceWindowId: str = field_defs["resourceMaintenanceWindowId"]
    clientId: str = field_defs["clientId"]
    clientName: str = field_defs["clientName"]
    source: str = field_defs["source"]
    externalAlertId: str = field_defs["externalAlertId"]
    comments: str = field_defs["comments"]


