from typing import Optional, List
from pydantic import BaseModel, Field, ValidationError

# define fields of resource object
field_defs = {
    "id" : Field(
            None, # default field to fetch
            title='id',
            description='Auto-generated 128-bit Version 4 UUID of object. '
                        'Conforms to https://tools.ietf.org/html/rfc4122.'
                        '**default field in any OpsQL queries**: yes.'
        ),
    "signatures": Field(
        ...,
        title='signatures',
        description='Signatures of the resource in the managed domain.'
                    'This is generated by the app as sha256 hashes'
                    'of one more combinations of resource attributes.'
                    'An app can generate multiple signatures for a resource'
                    'multiple signatures are useful to reconcile a resource'
                    'that is discovered by multiple apps'
                    'if a signature presented by an app matches that signatures associated'
                    'with an existing resource - then the newly discovered resource'
                    'is reconciled to the same resource'
    ),
    "state":   Field(
            None,
            title = 'managed',
            description='This is the manufacturer assigned serial for the resource.'
                        'E.g. FNRYZZ2, 000812345678300'
        ),
    "type" : Field(
        ...,
        title='type',
        description='type of a managed IT resource. '
                    'E.g. server, switch, storage_bucket.'
                    'default field in any OpsQL queries: yes.'
    ),
    "nativeType" : Field(
        ...,
        title='nativeType',
        description='type of a managed IT resource as defined within its domain.'
                    'E.g. AWS Elastic Load Balancer, GCP Cloud CDN, VMware vSphere Datastore.'
                    'default field in any OpsQL queries: yes.'
    ),
    "name" :  Field(
        ..., # default field to fetch
        title='name',
        description='Name of managed resource.'
                    'This is the name by which the resource is commonly identified.'
                    'E.g. for a server, this is the hostname, for a VM, this is the VM name.'
                    'Note: This field is fetched by OpsQL, by default.'
                    'default field in any OpsQL queries: yes.'
    ),
    "aliasName" : Field(
        None,
        title='aliasName',
        description='An alias name for the managed resource.'
                    'This is an alternate name by which the resource is also identified'
                    'This can be set by the user.'
    ),
    "dnsName" :  Field(
        None,
        title='dnsName',
        description='This is the name of the resource as reflected in the DNS system in the environment'
                    'This is also commonly referred to as fully qualified domain name (FQDN). E.g server-abc.mysubdomain.domain.com'
                    'See https://tools.ietf.org/html/rfc1594 and https://tools.ietf.org/html/rfc1035'
    ),
    "ipAddress" : Field(
        None,
        title='ipAddress',
        description='This is the IP address of the resource in IPv4 or IPv6 format'
                    'This is currently a single-value field.'
                    'i.e. a resource can have only one associated IP address'
                    'If a resource has multiple IP addresses'
                    'apps should use this field to denote the IP address'
                    'through which the app connects to the resource'
    ),
    "alternateIpAddress" : Field(
        None,
        title='alternateIpAddress',
        description='This is the an alternate IP address associated with the resource - in IPv4 or IPv6 format'
                    'This is intended to represent the NAT assigned IP of a resource, where NAT is used'
                    'See https://tools.ietf.org/html/rfc2663 for reference.'
    ),
    "macAddress" : Field(
        None,
        title='macAddress',
        description='This is Media Access Control address of the resource.'
                    'See https://standards.ieee.org for reference.'
    ),
    "os" : Field(
        None,
        title='os',
        description='This is the operating system running in the resource .'
                    'E.g. Ubuntu 20.04.1 LTS, Microsoft Windows Server 2019, Cisco IOS 15'
    ),
    "make" : Field(
        None,
        title='make',
        description='This is the manufacturer of the resource.'
                    'E.g. Microsoft Corporation, Dell, VMware, Inc., AWS, Azure, Google'
    ),
    "model" :  Field(
        None,
        title='model',
        description='This is the manufacturer assigned model of the resource.'
                    'E.g. PowerEdge M630 (Dell server), t2.large (AWS EC2 instance)'
    ),
    "serialNumber" :  Field(
        None,
        title='serialNumber',
        description='This is the manufacturer assigned serial for the resource.'
                    'E.g. FNRYZZ2, 000812345678300'
    )
}

# This is not a domain object. This class represents that IT resource discovered by the app.
# The attributes of the discovered resource as reconciled with that of existing domain objects of type Resource.
# If there is no existing Resource matches the signature of the discovered resource, a new Resource is created.
# If there is an existing Resource matches the signature of the discovered resource, the existing Resource is updated.
class DiscoveredResource(BaseModel):
    signatures: List[str] = field_defs["signatures"]
    type: str = field_defs["type"]
    nativeType: str = field_defs["nativeType"]
    name: str = field_defs["name"]
    aliasName: Optional[str] = field_defs["aliasName"]
    dnsName: Optional[str] = field_defs["dnsName"]
    ipAddress: Optional[str] = field_defs["ipAddress"]
    alternateIpAddress: Optional[str] = field_defs["alternateIpAddress"]
    macAddress: Optional[str] = field_defs["macAddress"]
    os: Optional[str] = field_defs["os"]
    make: Optional[str] = field_defs["make"]
    model: Optional[str] = field_defs["model"]
    serialNumber: Optional[str] = field_defs["serialNumber"]


# The class is the domain object that represents the managed resource in OpsRamp.
# This is what you get when you do GET /resources/{id}
class Resource(BaseModel):
    id : str = field_defs["id"]
    state: Optional[str] = field_defs["state"]
    signatures: List[str] = field_defs["signatures"]
    type: str = field_defs["type"]
    nativeType: str = field_defs["nativeType"]
    name: str = field_defs["name"]
    aliasName: Optional[str] = field_defs["aliasName"]
    dnsName: Optional[str] = field_defs["dnsName"]
    ipAddress: Optional[str] = field_defs["ipAddress"]
    alternateIpAddress: Optional[str] = field_defs["alternateIpAddress"]
    macAddress: Optional[str] = field_defs["macAddress"]
    os: Optional[str] = field_defs["os"]
    make: Optional[str] = field_defs["make"]
    model: Optional[str] = field_defs["model"]
    serialNumber: Optional[str] = field_defs["serialNumber"]

    def update(self, discovered_resource: DiscoveredResource):
        attributes = discovered_resource.dict()
        for k, v in attributes.items():
            if k in self.__dict__.keys():
                vars(self)[k] = v
            else:
                error_message = 'error: field {field} does not exists'.format(field=k)
                raise Exception(error_message)
                # print("error: key does not exists", key)


